from authens import views as authens_views
from django.contrib import messages
from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.auth.models import Group
from django.contrib.auth.views import PasswordChangeView
from django.core.exceptions import ImproperlyConfigured
from django.forms import ModelForm
from django.http import HttpResponseRedirect
from django.urls import reverse_lazy
from django.utils.translation import gettext_lazy as _
from django.views.generic import FormView, TemplateView

from shared.mixins import LocalUserOnlySoftMixin

from .forms import (
    BecomeErnestoForm,
    ProfileCreateForm,
    ProfileForm,
    UserCreateForm,
    UserForm,
)
from .models import Profile


# Create your views here.
class LogoutView(authens_views.LogoutView):
    def get_success_url(self):
        messages.success(self.request, _("Déconnecté avec succès"))
        return super().get_success_url()


class PasswordChange(LocalUserOnlySoftMixin, PasswordChangeView):
    redirect_url = reverse_lazy("ernestoprofil:edit-user")
    reject_message = _(
        "Vous ne pouvez pas changer votre mote de pass avec un compte CAS"
    )

    extra_context = {
        "title": _("Changer mon mot de passe"),
        "submit": _("Enregistrer"),
    }
    template_name = "simple_form.html"
    success_url = reverse_lazy("ernestoprofil:edit-user")


class BecomeErnesto(LoginRequiredMixin, FormView):
    template_name = "simple_form.html"
    form_class = BecomeErnestoForm
    success_url = reverse_lazy("home")

    def get_context_data(self, **kwargs):
        ctx = super().get_context_data(**kwargs)
        ctx.update(
            {
                "title": _("Devenir fanfaron"),
                "submit": _("Pouêt 🎶"),
            }
        )
        return ctx

    def form_valid(self, form):
        group, created = Group.objects.get_or_create(name="Ernestophoniste")
        group.user_set.add(self.request.user)
        messages.success(
            self.request, _("Bienvenue à la fanfare jeune ernestophoniste! 🎺")
        )
        return super().form_valid(form)


class BaseEditUser(TemplateView):
    def get_instance(self, fname):
        raise ImproperlyConfigured("")

    def get_context_data(self, **kwargs):
        inst_forms = self.get_forms()
        for f in self.forms:
            if f not in kwargs:
                kwargs[f] = inst_forms[f]
        return super().get_context_data(**kwargs)

    def get_success_url(self):
        return str(self.success_url)

    def get_forms(self):
        kwargs = {}
        if self.request.method in ("POST", "PUT"):
            kwargs.update(
                {
                    "data": self.request.POST,
                    "files": self.request.FILES,
                }
            )
        inst_forms = {}
        for f, form_class in self.forms.items():
            if issubclass(form_class, ModelForm):
                inst_forms[f] = form_class(
                    prefix=f,
                    instance=self.get_instance(f),
                    **kwargs,
                )

            else:
                inst_forms[f] = form_class(prefix=f, **kwargs)
        return inst_forms

    def valid(self, forms):
        pass

    def post(self, request, *args, **kwargs):
        """
        Handle POST requests: instantiate a form instance with the passed
        POST variables and then check if it's valid.
        """
        forms = self.get_forms()
        valid = True
        for fname, form in forms.items():
            valid &= form.is_valid()
        if valid:
            user = forms["user_form"].save()
            forms["profile_form"].instance.user = user
            print(forms["profile_form"].instance.polls_pseudo)
            for fname, form in forms.items():
                if isinstance(form, ModelForm):
                    form.save()
            self.valid(forms)
            return HttpResponseRedirect(self.get_success_url())
        else:
            return self.render_to_response(self.get_context_data(**forms))


class EditUser(LoginRequiredMixin, BaseEditUser):
    template_name = "ernestoprofile/edit-user.html"
    forms = {
        "user_form": UserForm,
        "profile_form": ProfileForm,
    }
    success_url = reverse_lazy("ernestoprofil:edit-user")

    def get_instance(self, fname):
        match fname:
            case "user_form":
                return self.request.user
            case "profile_form":
                if not hasattr(self.request.user, "profile"):
                    Profile.objects.create(
                        user=self.request.user, polls_pseudo=self.request.user.username
                    )
                return self.request.user.profile

    def get_success_url(self):
        messages.success(self.request, _("Profil mis à jour avec succès"))
        return super().get_success_url()


class CreateUser(BaseEditUser):
    template_name = "ernestoprofile/create-user.html"
    forms = {
        "user_form": UserCreateForm,
        "profile_form": ProfileCreateForm,
        "ernesto_form": BecomeErnestoForm,
    }
    success_url = reverse_lazy("home")

    def get_instance(self, fname):
        return None

    def valid(self, forms):
        user = forms["user_form"].instance
        group, created = Group.objects.get_or_create(name="Ernestophoniste")
        group.user_set.add(user)

    def get_success_url(self):
        messages.success(
            self.request,
            _(
                "Bienvenue chèr.re fanfaron.ne! Tu peux maintenant te connecter pour découvrir ce merveilleux site."
            ),
        )  # TODO middot
        return super().get_success_url()
